import pygame,model,time,random
#定义需要的常量
SCREEN_SIZE =(512,768)
SCREEN_RECT =pygame.Rect(0,0,*SCREEN_SIZE)
ENEMY_CREATE = pygame.USEREVENT
ENEMY_BOOM_SCORE = 10
ENEMY_BIGBOOM_SCORE = 4

pygame.font.init()
#初始化音效
pygame.mixer.init()
pygame.mixer.music.load("./mp3/menm.mp3")
#初始化音效
pygame.mixer.init()
soundwav = pygame.mixer.Sound("./mp3/da.wav")  # filename.wav文件名
#
pygame.mixer.init()
soundwavs = pygame.mixer.Sound("./mp3/5972.wav")  # filename.wav文件名
#
pygame.mixer.init()
soundwavss = pygame.mixer.Sound("./mp3/6319.wav")  # filename.wav文件名
# 设置音量
pygame.mixer.music.set_volume(50)
pygame.mixer.music.play(1)


# sound = pygame.mixer.Sound('')

class GameEngine:
    def __init__(self):

        # 定义英雄飞机对象
        self.hero = model.HeroSprite("./images/hero_1.png")
        self.hero1 = model.HeroSprite("./images/hero_2.png")
        pygame.display.set_caption("平平飞机大战")

        # 定义一个敌方飞机对象
        # 定义一个敌方大飞机对象
        self.enemy= model.EnemySprite("./images/p02-1.png")
        self.wushi =model.Boos("./images/boss2.png", hp1=20)
        self.bigenemy = model.EnemySprite("./images/big1.png")
         # 定义一个敌人飞机的精灵组对象
        # 定义一个敌人大飞机的精灵组对象
        self.enemys = pygame.sprite.Group()
        self.bigenemys = pygame.sprite.Group()
        self.bosse = pygame.sprite.Group()


        #定义一个补给子弹精灵组
        #定义一个补给血量精灵组
        self.bujib = pygame.sprite.Group()
        self.bujih = pygame.sprite.Group()

        # 爆炸敌机精灵组
        self.enemys_boom = pygame.sprite.Group()

        #定义爆炸飞机字典
        # 定义爆炸大飞机字典
        self.enemys_boom_dict = dict()
        self.bigenemys_boom_dict = dict()

        #创建一个屏幕变量
        self.screen = pygame.display.set_mode(SCREEN_SIZE)

        #引入字体
        self.game_font = pygame.font.SysFont("fangsong",16,True)

        #大飞机生命
        self.ssss = 10
    def show_index(self):
        pygame.init()
        # 添加一个时钟
        clock = pygame.time.Clock()
        while True:
            clock.tick(66)
            bgg_1 = pygame.image.load("./images/bgg2.jpg")
            self.screen.blit(bgg_1,(0,0))
            pygame.display.update()
            event_list = pygame.event.get()
            shubiao = list(pygame.mouse.get_pos())
            for event in event_list:
                if 110 < shubiao[0] < 400 and 504 < shubiao[1] < 616:
                    if event.type == pygame.MOUSEBUTTONUP:
                        self.start()

                if event.type == pygame.QUIT:
                    pygame.quit()
                    exit()


    def start(self):
        pygame.init()
        # 添加一个时钟
        clock = pygame.time.Clock()
        # 间隔一定的时间，触发一个创建敌机的事件
        pygame.time.set_timer(ENEMY_CREATE, 2000)
        self.create_scence()
        self.bg()
        self.bg22()
        self.bg33()

        while True:
            if  self.hero.score<100:
                clock.tick(24)
                self._check_collide()
                self._check_event()
                self.create_ran()
                self.check_kewdown()
                # 精灵组渲染
                self.resources.update()
                self.resources.draw(self.screen)
                self.update()
                pygame.display.update()

            elif self.hero.score>=100 and self.hero.score<1200:
                clock.tick(24)
                self._check_collide()
                self._check_event()
                self.create_ran()
                self.check_kewdown()

                # 精灵组渲染
                self.resources2.update()
                self.resources2.draw(self.screen)
                self.update()

                pygame.display.update()
    def create_scence(self):
        self.screen = pygame.display.set_mode(SCREEN_SIZE)

    def _check_event(self):

        #添加监听程序
        event_list = pygame.event.get()
        if len(event_list)>0:
            print(event_list)
            for event in event_list:
                print(event.type, pygame.KEYDOWN, pygame.K_LEFT)
                if event.type == pygame.QUIT:
                    pygame.quit()
                    exit()
                #创建敌方小飞机
                if event.type == ENEMY_CREATE:
                    print("创建一家敌方飞机。。。。。")
                    self.enemy = model.EnemySprite("./images/p02-1.png")
                    self.enemys.add(self.enemy)
            self.enemy.send_bullet()
##################################################33

##########################################################3
# 获取键盘的值
    def check_kewdown(self):
        key_down = pygame.key.get_pressed()
        if key_down[pygame.K_LEFT]:
            print("向左移动<<<<<<<<<<")
            self.hero.rect.x -=8
        if key_down[pygame.K_RIGHT]:
            print("向右移动>>>>>>>>>>>")
            self.hero.rect.x += 8
        if key_down[pygame.K_UP]:
            print("向上移^^^^^^^^^^^^^^^")
            self.hero.rect.y -=8
        if key_down[pygame.K_DOWN]:
            print("向下移动vvvvvvvvvvvv")
            self.hero.rect.y +=8
        if  key_down[pygame.K_END]:
            self.hero.fire()
            soundwavs.play()
            print("开火",self.hero.bullets)
        if key_down[pygame.K_a]:
            print("向左移动<<<<<<<<<<")
            self.hero1.rect.x -= 8
        if key_down[pygame.K_d]:
            print("向右移动>>>>>>>>>>>")
            self.hero1.rect.x += 8
        if key_down[pygame.K_w]:
            print("向上移^^^^^^^^^^^^^^^")
            self.hero1.rect.y -= 8
        if key_down[pygame.K_s]:
            print("向下移动vvvvvvvvvvvv")
            self.hero1.rect.y += 8

        elif key_down[pygame.K_f]:
            self.hero1.fire3()
            soundwavs.play()
            print("开火", self.hero1.bullets)

    def boom(self, enemy):
        for i in range(2, 6):
            clock = pygame.time.Clock()
            clock.tick(60)
            image = pygame.image.load("./enemy2_down_" + str(i) + ".png")
            self.screen.blit(image, enemy.rect)
            pygame.display.update()
            print(image)

    def _check_collide(self):
        # 碰撞检测:子弹和敌方飞机之间的碰撞！
        self.enemys_boom_dict= pygame.sprite.groupcollide(self.hero.bullets, self.enemys , True, True)

        #计算得分
        self.hero.score += len(self.enemys_boom_dict)*ENEMY_BOOM_SCORE
        self.enemys_boom.add(self.enemys_boom_dict)  # 添加到爆炸精灵组
        for enemy_boom in self.enemys_boom:
            self.boom(enemy_boom) # 将敌机返回到爆炸组，调用函数
            soundwavss.play()
            print("爆炸")
            self.enemys_boom.remove(enemy_boom)
            self.wu_shi()
        ###########################################################3
        # 碰撞检测:子弹和敌方大飞机之间的碰撞！
        self.bigenemys_boom_dict = pygame.sprite.groupcollide(self.hero.bullets, self.bigenemys, True,True)
        # 计算得分
        self.hero.score += len(self.bigenemys_boom_dict) * ENEMY_BIGBOOM_SCORE
        # 添加到爆炸精灵
        self.enemys_boom.add(self.bigenemys_boom_dict)
        for enemy_boom in self.enemys_boom:
            self.boom(enemy_boom)  # 将敌机返回到爆炸组，调用函数
            soundwavss.play()
            print("爆炸")
            self.enemys_boom.remove(enemy_boom)
        o = pygame.sprite.groupcollide(self.hero1.bullets, self.enemys, True,True)
        if len(o)>0:
            self.enemy.kill()

        #飞机与敌方飞机子弹碰撞
        if pygame.sprite.spritecollide(self.hero and self.hero1,self.enemy.bulletlist,True):
            self.hero.life =self.hero.life-1
            if self.hero.life <=0:
                self.hero.kill()
                self.game_over()
                pygame.quit()
                exit()

        # 碰撞检测：英雄飞机和敌方飞机之间的碰撞
        if pygame.sprite.spritecollide(self.hero and self.hero1, self.enemys, True):
            self.hero.life = self.hero.life - 1
            if self.hero.life <= 0:
                self.hero.kill()
                self.game_over()

        #补给与飞机碰撞
        pe = pygame.sprite.spritecollide(self.hero ,self.bujib,True)
        if len(pe) > 0:
            soundwav.play()
            self.buji.kill()
            self.hero.fire2()
            print("开火", self.hero.bullets)

        #血量补给与飞机碰撞
        if pygame.sprite.spritecollide(self.hero1, self.bujih, True):
            self.hero.life = self.hero.life + 1
            if self.hero.life>10:
                self.hero.life =10

        # 子弹与巫师碰撞
        res2 = pygame.sprite.groupcollide(self.bosse, self.hero.bullets, False, True)
        if len(res2) > 0:
            self.wushi.hp1 -= 1
            if self.wushi.hp1 <= 0:
                self.wushi.kill()


    def update(self):
 #子弹精灵组渲染
        self.hero.bullets.update()
        self.hero.bullets.draw(self.screen)

        self.hero1.bullets.update()
        self.hero1.bullets.draw(self.screen)

        self.bosse.update()
        self.bosse.draw(self.screen)

        # 敌机子弹渲染
        for self.enemy in self.enemys:
            self.enemy.bulletlist.update()
            self.enemy.bulletlist.draw(self.screen)
#渲染补给
        self.bujib.update()
        self.bujib.draw(self.screen)
# 渲染补给
        self.bujih.update()
        self.bujih.draw(self.screen)
#渲染敌机精灵组中所有飞机
        self.enemys.update()
        self.enemys.draw(self.screen)

        self.screen.blit(self.game_font.render(u"当前分数：%s" % self.hero.score, True, [0, 0, 255]), [20, 20])
        self.screen.blit(self.game_font.render(u"当前生命：%s" % self.hero.life, True, [255, 0, 0]), [400, 20])
        ##################

    #####################################
    # 创建随机生成对象方法
    def create_ran(self):
        c = random.randint(1, 1000)
        # 创建一个敌方大飞机
        if c < 15:
            print("创建一家敌方大飞机。。。。。")
            self.enemyss = model.EnemySprite("./images/big1.png")
            self.enemys.add(self.enemyss)

        elif c > 20 and c < 30:
            print("创建一个补给")
            self.buji = model.buji()
            self.bujib.add(self.buji)

        elif c > 990:
            print("创建一个补给")
            self.buji1 = model.buji1()
            self.bujih.add(self.buji1)
#####################################
    def bg(self):
        # 定义背景精灵组
        self.bg1 = model.BackGround("./images/bg-1.jpg")
        self.bg2 = model.BackGround("./images/bg-1.jpg", next=True)
        # 添加精灵组
        self.resources = pygame.sprite.Group(self.bg1, self.bg2, self.hero,self.hero1)

    # 创建背景对象
    def bg22(self):
        # 定义背景精灵组
        self.bg3 = model.BackGround("./images/bgimage1.jpg")
        self.bg4 = model.BackGround("./images/bgimage1.jpg", next=True)
        # 添加精灵组
        self.resources2 = pygame.sprite.Group(self.bg3, self.bg4, self.hero,self.hero1)

    def bg33(self):
        # 定义背景精灵组
        self.bg5 = model.BackGround("./images/imgbg_4.jpg")
        self.bg6 = model.BackGround("./images/imgbg_4.jpg", next=True)
        # 添加精灵组
        self.resources3 = pygame.sprite.Group(self.bg5, self.bg6, self.hero,self.hero1)

##############################################3

    def game_over(self):
        '''结束'''
        self.create_scence()
        while True:
            self.update()
            self.screen.blit(self.game_font.render("GAME OVER", True, [0, 255, 255]), [80, 200])
            # pygame.time.delay(1000)
            self.screen.blit(self.game_font.render('游戏得分：%s' % self.hero.score, True, [0, 255, 255]), [80, 300])
            # pygame.time.delay(1000)
            self.screen.blit(self.game_font.render("空格继续", True, [0, 255, 255]), [100, 400])

            pygame.display.update()
            event_list = pygame.event.get()
            if event_list:
                for event in event_list:
                    if event.type == pygame.KEYDOWN:
                        if event.key == pygame.K_SPACE:
                            # return True
                            # self.show()
                            break
            time.sleep(4)
            break
        pygame.quit()
        exit()
    def wu_shi(self):
        print(self.hero.score)
        if self.hero.score == 120:
            self.bosse.add(self.wushi)

        if self.hero.score == 180:
            self.wushi = model.Boos("./images/boss.png",hp1=40)
            self.bosse.add(self.wushi)








import pygame,random,math
#定义需要的常量
SCREEN_SIZE =(512,768)
SCREEN_RECT =pygame.Rect(0,0,*SCREEN_SIZE)
# 自定义一个事件
ENEMY_CREATE = pygame.USEREVENT
ENEMY_CREATES = pygame.USEREVENT


#定义游戏父类型主类
class GameSprite(pygame.sprite.Sprite):
    """游戏精灵对象：用于表示游戏中各种元素"""
    def __init__(self,image_path,speed=1):
        super().__init__()
        self.image = pygame.image.load(image_path)
        self.rect =self.image.get_rect()
        self.speed = speed

    def update(self):
        "，默认运动更新方法"
        self.rect.y += self.speed


#定义背景图片类型
class BackGround(GameSprite):
    def __init__(self,image_path,next=False):
        super().__init__(image_path,speed=1)
        #初始化边界
        if next:
            self.rect.y = -SCREEN_SIZE[1]

        #调用父类方法
        #更新
    def update(self):
        super().update()
        if self.rect.y > SCREEN_SIZE[1]:
            self.rect.y = -SCREEN_SIZE[1]

#定义我方飞机类型
class HeroSprite(GameSprite):
    def __init__(self,image_path):
        super().__init__(image_path,speed=0)
        # 初始生命值
        self.index = 0
        self.life = 10
        # 初始分值
        self.score = 0

        #初始化飞机位置
        self.rect.centerx = SCREEN_RECT.centerx
        self.rect.y = SCREEN_RECT.centery +200
        #创建一个子弹精灵组
        self.bullets = pygame.sprite.Group()
    #
    def update(self):
        #水平边界判断
        if self.rect.x <= 0:
            self.rect.x =0
        elif self.rect.x >= SCREEN_RECT.width-self.rect.width:
            self.rect.x = SCREEN_RECT.width-self.rect.width


        #垂直边界判断
        if self.rect.y<=0:
            self.rect.y = 0
        elif self.rect.y >= SCREEN_RECT.height -self.rect.height:
            self.rect.y = SCREEN_RECT.height -self.rect.height

    def fire(self):
        self.index +=20

        """飞机开火"""
        #创建一个子弹对象
        bullet = BulletSprite("./images/21.png",self.rect.centerx-15+30*math.cos(self.index),self.rect.centery-90)
        #添加到精灵组对象
        self.bullets.add(bullet)
    def fire3(self):
        self.index +=20

        """飞机开火"""
        #创建一个子弹对象
        bullet2 = BulletSprite("./images/bugizhi.png",self.rect.centerx-15+30*math.cos(self.index),self.rect.centery-90)
        #添加到精灵组对象
        self.bullets.add(bullet2)
    def fire2(self):
        """飞机开火"""
        #创建一个子弹对象

        bujibullet = BulletSprite3(self.rect.centerx-15,self.rect.centery-90)
        self.bullets.add(bujibullet)

#创建一个英雄飞机子弹精灵
class BulletSprite(GameSprite):
    """子弹精灵"""
    def __init__(self,image_path,x,y):
        super().__init__(image_path,speed =-8)
        self.rect.x = x
        self.rect.y = y

    def update(self):
        super().update()
        #边界判断
        if self.rect.y <= -self.rect.height:
            #子弹删除
            self.kill()
    def __del__(self):
        print("子弹对象已经销毁")


#创建一个英雄飞机补给子弹精灵
class BulletSprite3(GameSprite):
    """子弹精灵"""
    def __init__(self,x,y):
        super().__init__("./images/bujiji.png",speed =0)
        self.rect.x = SCREEN_RECT.centerx-230
        self.rect.y = SCREEN_RECT.y

    def update(self):
        super().update()
        #边界判断
        if self.rect.y <= -self.rect.height:
            #子弹删除
            self.kill()
    def __del__(self):
        print("子弹对象已经销毁")


#创建一个敌方飞机子弹精灵
class BulletSprite1(GameSprite):
    """子弹精灵"""
    def __init__(self,image_path,x,y):
        super().__init__(image_path,speed =6)
        self.rect.x = x
        self.rect.y = y

    def update(self):
        super().update()
        #边界判断
        if self.rect.y <= -SCREEN_RECT.height:
            #子弹删除
            self.kill()
    def __del__(self):
        print("子弹对象已经销毁")


#创建敌方飞机类型
class EnemySprite(GameSprite):
    def __init__(self,image_path):
        #初始化敌方飞机数据：图片，速度
        super().__init__(image_path,speed =random.randint(2,5))

        #初始化敌方飞机位置
        self.rect.x = random.randint (0,SCREEN_RECT.width-self.rect.width)
        self.rect.y = -self.rect.height
        self.bulletlist =pygame.sprite.Group()

    def send_bullet(self):
        e = random.randint(1,100)

         # 创建一个子弹对象
        if e >97:
            bullets = BulletSprite1("./images/33.png",self.rect.centerx - 31, self.rect.y)
            self.bulletlist.add(bullets)
    def send_bullet1(self):
         # 创建一个子弹对象
        bullets = BulletSprite1("./images/ez111.png",self.rect.centerx - 31, self.rect.y)
        self.bulletlist.add(bullets)

    def update(self):
        super().update()
        if self.rect.y > SCREEN_RECT.height:
            self.kill()

    def __del__(self):
        self.destroy()

    def destroy(self):
        print("敌机销毁")


##################################3

class buji(GameSprite):
    def __init__(self):
        super().__init__("./images/buji_1.png",speed = 5)
        #初始化补给位置
        self.rect.x = random.randint(0, SCREEN_RECT.width - self.rect.width)
        self.rect.y = -self.rect.height
    def update(self):
        super().update()
        if self.rect.y > SCREEN_RECT.height:
            self.kill()

    def __del__(self):
        self.destroy()

    def destroy(self):
        print("补给销毁")



class buji1(GameSprite):
    def __init__(self):
        super().__init__("./images/buji_2.png",speed = 3)
        #初始化补给位置
        self.rect.x = random.randint(0, SCREEN_RECT.width - self.rect.width)
        self.rect.y = -self.rect.height
    def update(self):
        super().update()
        if self.rect.y > SCREEN_RECT.height:
            self.kill()

    def __del__(self):
        self.destroy()

    def destroy(self):
        print("补给销毁")

######################################33
# 定义boss飞机类型
class Boos(GameSprite):
    '''巫师精灵对象'''

    def __init__(self, image_path,hp1):
        # 初始化巫师的图片、速度
        super().__init__(image_path, speed=10)
        # 初始化英雄飞机的位置
        self.rect.centerx = SCREEN_RECT.centerx  # width宽度
        self.rect.y = -self.rect.height
        self.bullets = pygame.sprite.Group()  # group组 bullets子弹
        self.skills = pygame.sprite.Group()
        self.hp1 = hp1
        self.speed2 = random.randint(1, 9)

    def update(self):
        # 调用父类的方法直接运动
        super().update()
        self.rect.x += self.speed2
        if self.rect.x >= SCREEN_RECT.width - self.rect.width:
            self.rect.x = SCREEN_RECT.width - self.rect.width
            self.speed2 = -self.speed2
            # self.speed = -self.speed
        if self.rect.x <= 0:
            self.rect.x = 0
            self.speed2 = -self.speed2
        if self.rect.y >= 0:
            self.rect.y = 0








#引入构建包信息的模块
from distutils.core import  setup

#定义发布包的文件的信息
setup(
    name ="weninide_fly",
    version= "1.0",
    description="飞机大战",
    author = "维尼熊",
    author_email = "wenini@163.com",
    py_modules = ['__init__','engine','font','main','model']
)